{ config, lib, pkgs, ... }:

let
  mainUserHome = "/home/palo";
in
{

  # grub configuration
  # ------------------
  boot.loader.grub = {
    device  = "/dev/sda";
    enable  = true;
    version = 2;
  };

  # lvm volume group
  # ----------------
  boot.initrd.luks.devices = [
    {
      name   = "vg";
      device = "/dev/sda2";
      preLVM = true;
    }
  ];

  # NTFS support
  # ------------
  environment.systemPackages = [
    pkgs.ntfs3g
  ];

  # root
  # ----
  fileSystems."/" = {
    options = [ "noatime" "nodiratime" "discard" ];
    device  = "/dev/vg/root";
    fsType  = "ext4";
  };

  # boot
  # ----
  fileSystems."/boot" = {
    device = "/dev/sda1";
    fsType = "ext4";
  };

  # home
  # ----
  fileSystems."/home" = {
    options   = [ "noatime" "nodiratime" ];
    device    = "/dev/mapper/decrypted_home";
    fsType    = "ext4";
    encrypted = {
      enable  = true;
      keyFile = "/mnt-root/root/keys/home.key";
      label   = "decrypted_home";
      blkDev  = "/dev/mapper/store-home";
    };
  };

  # var/lib/docker
  # --------------
  fileSystems."/var/lib/docker" = {
    options   = [ "noatime" "nodiratime" ];
    device    = "/dev/mapper/decrypted_docker";
    fsType    = "ext4";
    encrypted = {
      enable  = true;
      keyFile = "/mnt-root/root/keys/docker.key";
      label   = "decrypted_docker";
      blkDev  = "/dev/mapper/store-docker";
    };
  };

  # automount
  # ---------
  fileSystems."/media" = {
    device = "/dev/disk/by-uuid/f7fa1c0e-ac9f-4955-b4bd-644c1ddb0d89";
    fsType = "ext4";
    options = [
      "nofail"
      "noauto"
      #"x-systemd.device-timeout=1ms"
    ];
  };
  systemd.mounts = [
    {
      enable = true;
      options = "nofail,noauto";
      type = "ext4";
      wantedBy = ["multi-user.target"];
      what = "/dev/disk/by-uuid/f7fa1c0e-ac9f-4955-b4bd-644c1ddb0d89";
      where = "/media";
    }
  ];

}

