{ lib, config, pkgs, ... }:
{
  imports = [

    <backup-module>
    <krops-lib>
    <cluster-module>
    <modules>

    # needed
    <home-manager/nixos>

    # cross-compiling
    <cleverca22/qemu.nix>

    ./grub.nix
    ./tinc.nix
    ./sshd.nix
    ./sshd-known-hosts-private.nix
    ./sshd-known-hosts-public.nix
    ./sshd-known-hosts-bootup.nix
    ./packages.nix
    ./restic.nix
    ./syncthing.nix
    ./sftp-user.nix

    ./nginx.nix
    ./nginx-landingpage.nix
    ./networking-qos.nix

  ];


  # provide overlays
  # -----------------
  nixpkgs.overlays = [
    (import "${<pkgs>}")
    (import "${<nix-writers>}/pkgs")
  ];

  # allow un-free
  # -------------
  nixpkgs.config.allowUnfree = true;
  environment.variables.NIXPKGS_ALLOW_UNFREE = "1";

  # some system stuff
  # -----------------
  time.timeZone = "Europe/Berlin";
  i18n = {
    consoleFont   = "Lat2-Terminus16";
    consoleKeyMap = "us";
    defaultLocale = "en_US.UTF-8";
  };

  # swappiness
  # ----------
  # 0 = only when running out of RAM
  # 100 = always swapp
  boot.kernel.sysctl."vm.swappiness" = 0;


  # rewire NIX_PATH
  # ---------------
  environment.variables.NIX_PATH = lib.mkForce "/var/src";

  # Shell configuration
  # -------------------
  programs.custom = {
    bash.enable       = true;
    zsh.enable        = true;
  };

}
